
DECLARE

   v_anydata     ANYDATA;
   v_anytype     ANYTYPE;
   v_typecode    PLS_INTEGER;

   TYPE rt_typeinfo IS RECORD
   ( prec        PLS_INTEGER 
   , scale       PLS_INTEGER
   , len         PLS_INTEGER 
   , csid        PLS_INTEGER
   , csfrm       PLS_INTEGER
   , schema_name VARCHAR2(30)
   , type_name   VARCHAR2(30)
   , version     VARCHAR2(30)
   , count       PLS_INTEGER
   );
   r_typeinfo rt_typeinfo;

   TYPE rt_attrinfo IS RECORD
   ( prec           PLS_INTEGER
   , scale          PLS_INTEGER
   , len            PLS_INTEGER 
   , csid           PLS_INTEGER 
   , csfrm          PLS_INTEGER
   , attr_elt_type  ANYTYPE
   , aname          VARCHAR2(30)
   );
   r_attrinfo rt_attrinfo;

BEGIN

   /* Demonstration ANYDATA instance using DEPT_OT... */
   v_anydata := ANYDATA.ConvertObject(dept_ot(10,'SALES','DALLAS'));

   /* Get the ANYTYPE information for the ANYDATA instance... */
   v_typecode := v_anydata.GetType(v_anytype);
   DBMS_OUTPUT.PUT_LINE('-------------------------------------');
   DBMS_OUTPUT.PUT_LINE('Typecode   : ' || v_typecode);

   /* ANYTYPE type-level metadata... */
   v_typecode := v_anytype.GetInfo(r_typeinfo.prec,
                                   r_typeinfo.scale,
                                   r_typeinfo.len,
                                   r_typeinfo.csid,
                                   r_typeinfo.csfrm,
                                   r_typeinfo.schema_name,
                                   r_typeinfo.type_name,
                                   r_typeinfo.version,
                                   r_typeinfo.count);

   DBMS_OUTPUT.PUT_LINE('-------------------------------------');
   DBMS_OUTPUT.PUT_LINE('Typename   : ' || r_typeinfo.type_name);
   DBMS_OUTPUT.PUT_LINE('Attributes : ' || r_typeinfo.count);

   /* ANYTYPE attribute-level metadata... */
   FOR i IN 1 .. r_typeinfo.count LOOP
      v_typecode := v_anytype.GetAttrElemInfo(i,
                                              r_attrinfo.prec,
                                              r_attrinfo.scale,
                                              r_attrinfo.len,
                                              r_attrinfo.csid,
                                              r_attrinfo.csfrm,
                                              r_attrinfo.attr_elt_type,
                                              r_attrinfo.aname);
      DBMS_OUTPUT.PUT_LINE('-------------------------------------');
      DBMS_OUTPUT.PUT_LINE('Attribute  : ' || r_attrinfo.aname);
      DBMS_OUTPUT.PUT_LINE('Typecode   : ' || v_typecode);
      DBMS_OUTPUT.PUT_LINE('Length     : ' || NVL(r_attrinfo.len, r_attrinfo.prec));
   END LOOP;

END;
/

